"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const core_1 = require("@material-ui/core");
const formik_1 = require("formik");
const react_alert_1 = require("react-alert");
const components_1 = require("../../components");
const objects_1 = require("@hypernetlabs/objects");
const contexts_1 = require("../../contexts");
const CreateIdentityWidget_style_1 = require("../../widgets/CreateIdentityWidget/CreateIdentityWidget.style");
const CreateIdentityWidget = ({ onCloseCallback, registryName, currentAccountAddress, }) => {
    const alert = (0, react_alert_1.useAlert)();
    const classes = (0, CreateIdentityWidget_style_1.useStyles)();
    const { coreProxy } = (0, contexts_1.useStoreContext)();
    const { setLoading } = (0, contexts_1.useLayoutContext)();
    const handleError = (err) => {
        setLoading(false);
        alert.error((err === null || err === void 0 ? void 0 : err.message) || "Something went wrong!");
        onCloseCallback();
    };
    const handleCreateIdentity = ({ label, recipientAddress, tokenUri, }) => {
        setLoading(true);
        coreProxy
            .createRegistryEntry(registryName, label, (0, objects_1.EthereumAddress)(recipientAddress), tokenUri)
            .map(() => {
            setLoading(false);
            onCloseCallback();
        })
            .mapErr(handleError);
    };
    return (react_1.default.createElement(components_1.GovernanceDialog, { title: "Create a New Identity", isOpen: true, onClose: onCloseCallback, content: react_1.default.createElement(core_1.Box, { className: classes.wrapper },
            react_1.default.createElement(formik_1.Formik, { initialValues: {
                    label: "",
                    recipientAddress: currentAccountAddress,
                    tokenUri: "",
                }, onSubmit: handleCreateIdentity }, ({ handleSubmit, values }) => {
                return (react_1.default.createElement(formik_1.Form, { onSubmit: handleSubmit },
                    react_1.default.createElement(components_1.GovernanceField, { title: "Label", name: "label", type: "input", placeholder: "Enter a label" }),
                    react_1.default.createElement(components_1.GovernanceField, { title: "Recipient Address", required: true, name: "recipientAddress", type: "input", placeholder: "Enter the recipient address" }),
                    react_1.default.createElement(components_1.GovernanceField, { title: "Token URI", required: true, name: "tokenUri", type: "input", placeholder: "Enter the token URI" }),
                    react_1.default.createElement(core_1.Box, { className: classes.actionContainer },
                        react_1.default.createElement(components_1.GovernanceButton, { variant: "outlined", color: "primary", onClick: onCloseCallback }, "Cancel"),
                        react_1.default.createElement(components_1.GovernanceButton, { className: classes.button, variant: "contained", color: "primary", onClick: handleSubmit, disabled: !values.recipientAddress || !values.tokenUri }, "Submit"))));
            })) }));
};
exports.default = CreateIdentityWidget;
//# sourceMappingURL=CreateIdentityWidget.js.map