"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const core_1 = require("@material-ui/core");
const react_alert_1 = require("react-alert");
const components_1 = require("../../components");
const ProposalDetailWidget_style_1 = require("../../widgets/ProposalDetailWidget/ProposalDetailWidget.style");
const contexts_1 = require("../../contexts");
const objects_1 = require("@hypernetlabs/objects");
const ProposalDetailWidget = ({ onProposalListNavigate, proposalId, }) => {
    const classes = (0, ProposalDetailWidget_style_1.useStyles)();
    const alert = (0, react_alert_1.useAlert)();
    const { coreProxy } = (0, contexts_1.useStoreContext)();
    const { setLoading } = (0, contexts_1.useLayoutContext)();
    const [proposal, setProposal] = (0, react_1.useState)();
    const [accountAddress, setAccountAddress] = (0, react_1.useState)();
    const [supportStatus, setSupportStatus] = (0, react_1.useState)();
    (0, react_1.useEffect)(() => {
        getProposalDetail();
    }, []);
    const getProposalDetail = () => {
        setLoading(true);
        coreProxy.getEthereumAccounts().map((accounts) => {
            setAccountAddress(accounts[0]);
            // delegate votes, createProposal and then list all proposals
            coreProxy
                .getProposalDetails(proposalId)
                .map((proposal) => {
                setProposal(proposal);
            })
                .mapErr(handleError);
            getProposalVotesReceipt(accounts[0]);
        });
    };
    const getProposalVotesReceipt = (account) => {
        setLoading(true);
        coreProxy
            .getProposalVotesReceipt(proposalId, account)
            .map((proposalVoteReceipt) => {
            if (proposalVoteReceipt.hasVoted) {
                setSupportStatus(Number(proposalVoteReceipt.support));
            }
            setLoading(false);
        })
            .mapErr(handleError);
    };
    const handleError = (err) => {
        setLoading(false);
        alert.error((err === null || err === void 0 ? void 0 : err.message) || "Something went wrong!");
    };
    const proposalVotesFor = proposal ? proposal.votesFor : 0;
    const proposalVotesAgainst = proposal ? proposal.votesAgainst : 0;
    const proposalVotesAbstain = proposal ? proposal.votesAbstain : 0;
    const totalVotes = proposalVotesFor + proposalVotesAgainst + proposalVotesAbstain;
    const forPercentage = ((proposalVotesFor * 100) / totalVotes).toFixed(0) || "0";
    const againstPercentage = ((proposalVotesAgainst * 100) / totalVotes).toFixed(0) || "0";
    const abstainPercentage = ((proposalVotesAbstain * 100) / totalVotes).toFixed(0) || "0";
    const showVotingButtons = Number(proposal === null || proposal === void 0 ? void 0 : proposal.state) === objects_1.EProposalState.ACTIVE;
    const queueProposal = () => {
        setLoading(true);
        coreProxy
            .queueProposal(proposalId)
            .map((proposal) => {
            setProposal(proposal);
            setLoading(false);
        })
            .mapErr(handleError);
    };
    const cancelProposal = () => {
        setLoading(true);
        coreProxy
            .cancelProposal(proposalId)
            .map((proposal) => {
            setProposal(proposal);
            setLoading(false);
        })
            .mapErr(handleError);
    };
    const executeProposal = () => {
        setLoading(true);
        coreProxy
            .executeProposal(proposalId)
            .map((proposal) => {
            setProposal(proposal);
            setLoading(false);
        })
            .mapErr(handleError);
    };
    const castVote = (voteSupport) => {
        setLoading(true);
        coreProxy
            .castVote(proposalId, voteSupport)
            .map((proposal) => {
            setProposal(proposal);
            if (accountAddress) {
                getProposalVotesReceipt(accountAddress);
            }
        })
            .mapErr(handleError);
    };
    const isUserProposalOwner = (0, react_1.useMemo)(() => proposal &&
        accountAddress &&
        (0, objects_1.EthereumAddress)(proposal.originator) === accountAddress, [JSON.stringify(proposal), accountAddress]);
    const canQueueProposal = (0, react_1.useMemo)(() => {
        return Number(proposal === null || proposal === void 0 ? void 0 : proposal.state) === objects_1.EProposalState.SUCCEEDED;
    }, [JSON.stringify(proposal)]);
    const canExecuteProposal = (0, react_1.useMemo)(() => {
        return Number(proposal === null || proposal === void 0 ? void 0 : proposal.state) === objects_1.EProposalState.QUEUED;
    }, [JSON.stringify(proposal)]);
    const canCancelProposal = (0, react_1.useMemo)(() => {
        return (isUserProposalOwner &&
            [
                objects_1.EProposalState.PENDING,
                objects_1.EProposalState.QUEUED,
                objects_1.EProposalState.ACTIVE,
                objects_1.EProposalState.SUCCEEDED,
                objects_1.EProposalState.DEFEATED,
                objects_1.EProposalState.EXPIRED,
            ].includes(Number(proposal === null || proposal === void 0 ? void 0 : proposal.state)));
    }, [JSON.stringify(proposal), isUserProposalOwner]);
    const getHeaderActions = (0, react_1.useCallback)(() => {
        return [
            ...(canCancelProposal
                ? [
                    {
                        label: "Cancel Proposal",
                        onClick: () => {
                            cancelProposal();
                        },
                        variant: "outlined",
                        color: "secondary",
                    },
                ]
                : []),
            ...(canQueueProposal
                ? [
                    {
                        label: "Queue Proposal",
                        onClick: () => {
                            queueProposal();
                        },
                        variant: "outlined",
                    },
                ]
                : []),
            ...(canExecuteProposal
                ? [
                    {
                        label: "Execute Proposal",
                        onClick: () => {
                            executeProposal();
                        },
                        variant: "outlined",
                    },
                ]
                : []),
        ];
    }, [canQueueProposal, canExecuteProposal, canCancelProposal]);
    return (react_1.default.createElement(core_1.Box, null,
        react_1.default.createElement(components_1.GovernanceWidgetHeader, { label: "Proposal Detail", navigationLink: {
                label: "Proposal List",
                onClick: () => {
                    onProposalListNavigate === null || onProposalListNavigate === void 0 ? void 0 : onProposalListNavigate();
                },
            }, headerActions: getHeaderActions() }),
        (proposal === null || proposal === void 0 ? void 0 : proposal.state) != null && (react_1.default.createElement(core_1.Box, { className: classes.proposalStatus },
            react_1.default.createElement(components_1.GovernanceStatusTag, { status: proposal === null || proposal === void 0 ? void 0 : proposal.state }))),
        react_1.default.createElement(core_1.Box, { display: "flex" },
            react_1.default.createElement(core_1.Grid, { container: true, spacing: 3 },
                react_1.default.createElement(core_1.Grid, { item: true, xs: 4 },
                    react_1.default.createElement(components_1.GovernanceVotingCard, { type: "for", value: proposalVotesFor, progressValue: parseFloat(forPercentage), onVoteClick: () => castVote(objects_1.EProposalVoteSupport.FOR), isVoted: supportStatus === objects_1.EProposalVoteSupport.FOR, showVoteButton: showVotingButtons, disableVoteButton: Number(proposal === null || proposal === void 0 ? void 0 : proposal.state) !== objects_1.EProposalState.ACTIVE })),
                react_1.default.createElement(core_1.Grid, { item: true, xs: 4 },
                    react_1.default.createElement(components_1.GovernanceVotingCard, { type: "against", value: proposalVotesAgainst, progressValue: parseFloat(againstPercentage), onVoteClick: () => castVote(objects_1.EProposalVoteSupport.AGAINST), isVoted: supportStatus === objects_1.EProposalVoteSupport.AGAINST, showVoteButton: showVotingButtons, disableVoteButton: Number(proposal === null || proposal === void 0 ? void 0 : proposal.state) !== objects_1.EProposalState.ACTIVE })),
                react_1.default.createElement(core_1.Grid, { item: true, xs: 4 },
                    react_1.default.createElement(components_1.GovernanceVotingCard, { type: "abstain", value: proposalVotesAbstain, progressValue: parseFloat(abstainPercentage), onVoteClick: () => castVote(objects_1.EProposalVoteSupport.ABSTAIN), isVoted: supportStatus === objects_1.EProposalVoteSupport.ABSTAIN, showVoteButton: showVotingButtons, disableVoteButton: Number(proposal === null || proposal === void 0 ? void 0 : proposal.state) !== objects_1.EProposalState.ACTIVE })))),
        react_1.default.createElement(core_1.Box, { className: classes.proposalDetails },
            react_1.default.createElement(core_1.Typography, { variant: "h5", className: classes.proposalDetailsLabel }, "Details:"),
            react_1.default.createElement(core_1.Box, { className: classes.proposerSectionWrapper },
                react_1.default.createElement(core_1.Typography, { variant: "h5", className: classes.proposerLabel }, "Proposer"),
                react_1.default.createElement(core_1.Typography, { variant: "h5", className: classes.proposerValue }, proposal === null || proposal === void 0 ? void 0 : proposal.originator)),
            react_1.default.createElement(components_1.GovernanceMarkdown, { source: proposal === null || proposal === void 0 ? void 0 : proposal.description }))));
};
exports.default = ProposalDetailWidget;
//# sourceMappingURL=ProposalDetailWidget.js.map